/*************************************************************************************
	USBUA100SYSEX - sends system exclusive MIDI control messages to a Roland UA-100

	(C) 2001 by Michael Minn - michael@michaelminn.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

*************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

unsigned char convert_hex_character_to_byte(char hex_character)
{
	if ((hex_character >= '0') && (hex_character <= '9'))
		return ((unsigned char) hex_character) - ((unsigned char) '0');
	else if ((hex_character >= 'A') && (hex_character <= 'F'))
		return ((unsigned int) hex_character) + 10 - ((unsigned int) 'A');
	else if ((hex_character >= 'a') && (hex_character <= 'f'))
		return ((unsigned int) hex_character) + 10 - ((unsigned int) 'a');
	else
	{
		printf("Invalid hexadecimal character '%c'\n", hex_character);
		exit(-1);
	}
}

int convert_hex_string_to_bytes(char *hex_string, unsigned char *bytes)
{
	int length = 0;

	for (char *scan = hex_string; (scan != NULL) && (*scan != 0); scan += 2, ++length)
		bytes[length] = (convert_hex_character_to_byte(*scan) << 4) + convert_hex_character_to_byte(*(scan + 1));

	return length;
}

char *convert_byte_to_hex_string(unsigned char byte)
{
	static char string[4] = "   ";
	unsigned char lower_half = byte & 0x0f;
	unsigned char upper_half = (byte >> 4) & 0x0f;

	if (upper_half < 10)
		string[1] = '0' + upper_half;
	else
		string[1] = 'a' + upper_half - 10;

	if (lower_half < 10)
		string[2] = '0' + lower_half;
	else
		string[2] = 'a' + lower_half - 10;

	return string;
}

int display_message(unsigned char *message, int length)
{
	for (int scan = 0; scan < length; ++scan)
		printf("%s", convert_byte_to_hex_string(message[scan]));
	printf("\n");
	return length;
}


int main (int argc, char **argv)
{
	if (argc < 4)
	{
		printf("USBUA100SYSEX v2001.06.24 by Michael Minn\n\n");
		printf("SYNTAX: usbua100sysex <midi device name> <address> <data bytes>...\n");
		printf("    Sends system exclusive MIDI control messages to a Roland UA-100.\n");
		printf("    Example: usbua100sysex /dev/midi02  00405003  64\n");
		printf("             (changes parameter 00405003 (master volume level) to 64 (50%)\n");
		return -1;
	}

	int length = 6;
	unsigned char message[256] = { 0xF0, 0x41, 0x10, 0x00, 0x11, 0x12 };

	for (int argument = 2; argument < argc; ++argument)
		length += convert_hex_string_to_bytes(argv[argument], &message[length]);

	message[length] = 0;
	for (int checksum = 6; checksum < length; ++checksum)
		message[length] += message[checksum];

	message[length] = 128 - (message[length] & 0x7f);
	++length;
	message[length] = 0xf7;
	++length;

	int device = open(argv[1], O_RDWR);

	if (device < 0)
	{
		perror(argv[1]);
		return -1;
	}

	if (write(device, message, length) < length)
	{
		perror(argv[1]);
		return -1;
	}

	display_message(message, length);

	usleep(500000);

	return 0;
}


